;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - callback.ss                                                    #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Peter Ivanyi, 2007                                               #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################

(module mreddesigner-callback mzscheme
  (require (lib "file.ss")
           mzlib/list 
           (lib "class.ss")
           (lib "mred.ss" "mred")
           (lib "framework.ss" "framework")
           (file "graph-editor/graph-editor.ss")
           (file "mreddesigner-misc.ss")
           (file "mreddesigner-widgets.ss")
           (file "mreddesigner-callback-edit.ss")
           (file "mreddesigner-default-values.ss") ; for changed flag
  )
  
  ; contains a list of widgets as a pair
  ; in the pair the first element is the id, the second element is the type
  (define widget-list '())
  (define graph-editor-list-box #f)
  (provide graph-editor-canvas)
  (define graph-editor-canvas #f)
  
  ; function to delete everything from the editor
  (provide graph-widget-list-empty)
  (define (graph-widget-list-empty)
    (set! widget-list '())
    (send graph-editor-list-box clear)
    (send graph-editor-canvas clear)
    (changed! #f)
  )
  
  ; make a widget selected
  (provide graph-widget-list-select)
  (define (graph-widget-list-select name)
    (send graph-editor-list-box set-string-selection name)
  )
  
  ; function to add a widget to the graph editor
  (provide graph-widget-list-add)
  (define (graph-widget-list-add id type)
    (if (assoc id widget-list)
      (error "Duplicate widget creation, " id)
      ; project and frames are not added
      (if (not (member type '(project frame)))
        (begin
          (set! widget-list (append widget-list (list (list id type))))
          (set! widget-list (sort widget-list 
                                  (lambda (x y) (string<? (car x) (car y)))))
          (send graph-editor-list-box set (map car widget-list))
          (changed! #f)
        )
      )
    )
  )
  
  ; function to remove a widget from the graph editor
  (provide graph-widget-list-del)
  (define (graph-widget-list-del id)
    (let
      ((type (send hierarchy-widget item-property-get id 'type #f)))
      (if (not (member type '(project frame)))
        (if (assoc id widget-list)
          (begin
            (set! widget-list (assoc-remove widget-list (list id)))
            (send graph-editor-list-box set (map car widget-list))
            ; force the deletion from the graph editor
            (let*
              ((node (send graph-editor-canvas node-get-by-name id)))
              (if node
                (send graph-editor-canvas node-del! node)
              )
            )
            (changed! #f)
          )
          (error "Widget is not in the list, cannot be deleted, " id)
        )
      )
    )
  )
  
  (provide graph-widget-list-change-id)
  (define (graph-widget-list-change-id type old-id new-id)
    (if (not (member type '(project frame)))
      (if (assoc old-id widget-list)
        (begin
          (set! widget-list (assoc-change widget-list old-id new-id))
          (send graph-editor-list-box set (map car widget-list))
          ; force the deletion from the graph editor
          (let*
            ((node (send graph-editor-canvas node-get-by-name old-id)))
            (if node
              (begin
                (send node set-name new-id)
                (changed! #f)
              )
            )
          )
        )
        (error "Widget is not in the list, cannot be changed, " old-id)
      )
    )
  )
  
  ; ----------------------------------------------------------------------------
  
  ; edit the possible action at a node
  ; node is always an action !
  (define (edit-action node)
    (let
      ((tab-in  (send node tab-in-ref 0))
       (tab-out (send node tab-out-ref 0)))
      ; check that there are connecting lines
      (if (and (send tab-in connected?)
               (send tab-out connected?))
        (let*
          ; get the lines and the nodes at the other end
          ((line-in  (car (send tab-in get-lines)))
           (src-tab  (send line-in get-source))
           (src-node (send src-tab get-node))
           (line-out (car (send tab-out get-lines)))
           (dst-tab  (send line-out get-target))
           (dst-node (send dst-tab get-node))
          )
          ; display a window where the user can modify the actions
          (edit-action-window node
                              (send src-node get-name)
                              (node-property-get src-node 'type)
                              (send dst-node get-name)
                              (node-property-get dst-node 'type)
          )
          #f
        )
      )
    )
  )
  
  ; ----------------------------------------------------------------------------
  
  ; timer for double clicking
  (define timer-state #f)
  (define timer (new timer%
                     (interval #f)
                     (just-once? #t)
                     (notify-callback (lambda ()
                                        (set! timer-state #f)))))
  
  ; editor callback function,
  ; this function is called when something happened in the graph editor
  (define (editor-callback action data)
    (changed! #f)
    (case action
      ((select)
       (if timer-state
         (let ((type (node-property-get data 'node-type)))
           (if (equal? type 'action)
             (edit-action data)
           )
         )
         (begin
           (set! timer-state #t)
           (send timer start 300 #t)
         )
       )
      )
      ((before-node-add)
       (let
         ((node (send graph-editor-canvas node-get-by-name data)))
         (if (and (not (equal? data "action"))
                  node)
           (begin
             (message-box "Error"
                          "Widget already exists in the network"
                          #f '(ok stop))
             #f
           )
           #t
         )
       )
      )
      ((before-line-add)
       (let*
         ((src (car data))
          (dst (cadr data))
          (node-src (send src get-node))
          (type-src (node-property-get node-src 'node-type))
          (node-dst (send dst get-node))
          (type-dst (node-property-get node-dst 'node-type))
         )
         (cond
           ((equal? type-src type-dst)
            (message-box "Error"
                          "Only widget and action can be connected to each other"
                          #f '(ok stop))
            #f
           )
           ((and (equal? type-src 'action)
                 (send src connected?))
             (message-box "Error"
                          "An action can have only one output"
                          #f '(ok stop))
             #f
           )
           (else #t)
         )
       )
      )
      ((before-line-del)
       (let*
         ((src (car data))
          (dst (cadr data))
          (node-src (send src get-node))
          (type-src (node-property-get node-src 'node-type))
          (node-dst (send dst get-node))
          (type-dst (node-property-get node-dst 'node-type))
         )
         (if (equal? type-src 'action)
           (node-property-set! node-src 'callback-action #f)
         )
         (if (equal? type-dst 'action)
           (node-property-set! node-dst 'callback-action #f)
         )
       )
       #t
      )
      (else #t)
    )
  )
  
  ; ----------------------------------------------------------------------------
  
  (define (add-action graph-editor name x y)
    (let*
      ((name (if name name (symbol->string (gensym "action-"))))
       (node (send graph-editor-canvas node-add name x y '()))
      )
      (node-property-make node)
      (node-property-set! node 'node-type 'action)
      node
    )
  )
  
  
  (define (add-widget graph-editor name type x y)
    (let*
      ((node   #f)
       (style  '())
       )
      ; only these widgets can have output
      (if (not (member type node-callbacks))
        (set! style '(no-output))
      )
      (set! node (send graph-editor node-add name x y style))
      (if node
        (begin
          (node-property-make node)
          (node-property-set! node 'node-type 'widget)
          (node-property-set! node 'type type)
          node
        )
        #f
      )
    )
  )
  
  ; ----------------------------------------------------------------------------
  
  (define (graph-write fp)
    (display "(\n" fp)
    ; nodes
    (display "  (\n" fp)
    (send graph-editor-canvas node-for-each
      (lambda (key node)
        (display "    (" fp)
        (display " " fp)
        (write (send node get-name) fp)
        (display " " fp)
        (display (send node get-x) fp)
        (display " " fp)
        (display (send node get-y) fp)
        ; extra data
        (display " " fp)
        (display (node-property-get node 'node-type) fp)
        (display " " fp)
        (display (node-property-get node 'type) fp)
        (display " " fp)
        (write (node-property-get node 'callback-action) fp)
        
        (display ")\n" fp)
      )
    )
    (display "  )\n" fp)
    ; lines
    (display "  (\n" fp)
    (send graph-editor-canvas node-for-each
      (lambda (key dst-node)
        (let
          ((n (send dst-node tab-in-count)))
          (do ((i 0 (+ i 1))) ((= i n))
            (let*
              ((tab-in   (send dst-node tab-in-ref i))
               (lines    (send tab-in get-lines)))
              (if (not (null? lines))
                (let*
                  ((line (car lines))
                   (src-tab  (send line get-source))
                   (src-node (send src-tab get-node))
                  )
                  (display "    (" fp)
                  (write (send src-node get-name) fp)
                  (display " " fp)
                  (write (send dst-node get-name) fp)
                  (display ")\n" fp)
                )
              )
            )
          )
        )
      )
    )
    (display "  )\n" fp)
    (display ")\n" fp)
  )

  
  (provide callback-graph-write)
  (define (callback-graph-write file-name)
    (let*
      ((name (path-replace-suffix file-name ".net")))
      (call-with-output-file name
        (lambda (p-out)
          (fprintf p-out "~s\n" '(*mred-graph-2*))
          (graph-write p-out)
        )
        'replace
      )
    )
  )
  
  
  (define (graph-read lst func-node-read)
    (let
      ((nodes (car lst))
       (lines (cadr lst))
      )
      ; nodes
      (for-each
        (lambda (node)
          (let
            ((name      (list-ref node 0))
             (x         (list-ref node 1))
             (y         (list-ref node 2))
             (node-type (list-ref node 3))
             (type      (list-ref node 4))
             (action    (list-ref node 5))
            )
            (cond
              ((equal? node-type 'widget)
               (add-widget graph-editor-canvas name type x y)
              )
              ((equal? node-type 'action)
               (let ((node (add-action graph-editor-canvas name x y)))
                 (if action
                   (node-property-set! node 'callback-action action)
                 )
               )
              )
            )
          )
        )
        nodes
      )
      ; lines
      (for-each
        (lambda (line)
          (let*
            ((src-name (list-ref line 0))
             (src-node (send graph-editor-canvas node-get-by-name src-name))
             (dst-name (list-ref line 1))
             (dst-node (send graph-editor-canvas node-get-by-name dst-name))
             (n        (send dst-node tab-in-count))
             (ok?      #f)
            )
            (if (= n 0)
              (send dst-node tab-in-incr)
            )
            (do ((i 0 (+ i 1))) ((or ok? (= i n)))
              (let
                ((tab-in (send dst-node tab-in-ref i)))
                (if (not (send tab-in connected?))
                  (set! ok? tab-in)
                )
              )
            )
            (if (not ok?)
              (send dst-node tab-in-incr)
            )
            (let
              ((dst-tab (send dst-node tab-in-ref (- (send dst-node tab-in-count) 1)))
               (src-tab (send src-node tab-out-ref 0))
              )
              (send graph-editor-canvas line-add src-tab dst-tab)
            )
          )
        )
        lines
      )
    )
  )
  
  
  (provide callback-graph-read)
  (define (callback-graph-read file-name)
    (let*
      ((name (path-replace-suffix file-name ".net")))
      (if (file-exists? name)
        (call-with-input-file name
          (lambda (p-in)
            (if (equal? (read p-in) '(*mred-graph-2*))
              (begin
                (let 
                  ((new-lst 
                     (with-handlers 
                       ((exn:fail? (lambda (exn)
                                     (message-box "Error" 
                                                  (string-append "Error while reading the graph file: "
                                                                 (exn-message exn))
                                                  #f '(ok stop))
                                     #f)))
                       (read p-in))))
                  (if new-lst
                    (graph-read new-lst #f)
                  )
                )
              )
              (message-box "Error" "The graph file format is not valid." #f '(ok stop))))))))
  
  ; --------------------------------------------------------------------------------------
  
  (provide callback-graph-frame-create)
  (define (callback-graph-frame-create window)
   (letrec ((group-box (new group-box-panel% (label "") (parent window)))
            (horizontal-panel-group (new panel:horizontal-dragable% 
                                         (border 3) 
                                         (parent group-box)))
            (vertical-panel-control
              (new
               vertical-panel%
               (parent horizontal-panel-group)
               (alignment '(left top))
               (stretchable-width #f)))
            (message-widgets (new message% 
                                  (parent vertical-panel-control) 
                                  (label "Widgets")))
            (widget-choice
              (new
               list-box%
               (parent vertical-panel-control)
               (choices '())
               (min-width 150)
               (label #f)
               (callback 
                 (lambda (lb e)
                   (let ((type (send e get-event-type)))
                     (cond
                       ((equal? type 'list-box-dclick)
                        (let*
                          ((widget (send widget-choice get-string-selection)))
                          (if widget
                            (add-widget graph-editor-canvas 
                                        widget
                                        (cadr (assoc widget widget-list)) 
                                        -1 -1)
                          )
                        )
                       )
                       ((equal? type 'list-box)
                        (let*
                          ((widget (send widget-choice get-string-selection)))
                          (if widget
                            (send hierarchy-widget set-selected widget)
                          )
                        )
                       )
                     )
                   )
                 )
               )))
            (button-add-widget
              (new button% 
                   (parent vertical-panel-control) 
                   (label "Add widget") 
                   (stretchable-width #t)
                   (callback
                     (lambda (b e)
                       (let*
                         ((widget (send widget-choice get-string-selection)))
                         (if widget
                           (add-widget graph-editor-canvas 
                                       widget
                                       (cadr (assoc widget widget-list))
                                       -1 -1)
                         )
                       )
                     )
                   )
                   ))
            (button-add-action
              (new button% 
                   (parent vertical-panel-control)
                   (label "Add action")
                   (stretchable-width #t)
                   (callback
                     (lambda (b e)
                       (add-action graph-editor-canvas #f -1 -1)
                     ))))
            (button-del
              (new button% 
                   (parent vertical-panel-control)
                   (label "Delete node")
                   (stretchable-width #t)
                   (callback
                     (lambda (b e)
                       (let
                         ((node (send graph-editor-canvas get-selected)))
                         (if node
                           (send graph-editor-canvas node-del node)
                         )
                       )
                     )
                   )))
            (button-add-input
              (new button%
                   (parent vertical-panel-control)
                   (label "Add input tab")
                   (stretchable-width #t)
                   (callback
                     (lambda (item event)
                       (let*
                         ((selected (send graph-editor-canvas get-selected)))
                         (if (and selected 
                                  (not (equal? (node-property-get selected 'node-type) 'action)))
                           (send selected tab-in-incr))))
                   )))
            (button-del-input
              (new button%
                   (parent vertical-panel-control)
                   (label "Del input tab")
                   (stretchable-width #t)
                   (callback
                     (lambda (item event)
                       (let*
                         ((selected (send graph-editor-canvas get-selected)))
                         (if (and selected 
                                  (not (equal? (node-property-get selected 'node-type) 'action)))
                           (send selected tab-in-decr))))
                   )))
            
            ; menu
            (window-menu (new menu-bar% (parent window)))
            (file-menu (new menu%
                            (label "File") 
                            (parent window-menu)))
            (new-menu  (new menu-item%
                            (label "&New graph...")
                            (shortcut #\N)
                            (parent file-menu)
                            (callback (lambda (item event)
                                        ; clear only the graph
                                        (send graph-editor-canvas clear)
                                        ))))
            (close-menu  (new menu-item%
                              (label "&Close window")
                              (shortcut #\C)
                              (parent file-menu)
                              (callback (lambda (item event)
                                          (send window show #f)
                                          (send window on-close)
                                          ))))
            (edit-menu (new menu%
                            (label "Edit") 
                            (parent window-menu)))
            (add-input-tab-menu  (new menu-item%
                                      (label "Add input tab")
                                      (parent edit-menu)
                                      (callback 
                                        (lambda (item event)
                                          (let*
                                            ((selected (send graph-editor-canvas get-selected)))
                                            (if (and selected 
                                                     (not (equal? (node-property-get selected 'node-type) 'action)))
                                              (send selected tab-in-incr)))))))
            (del-input-tab-menu  (new menu-item%
                                      (label "Remove input tab")
                                      (parent edit-menu)
                                      (callback 
                                        (lambda (item event)
                                          (let*
                                            ((selected (send graph-editor-canvas get-selected)))
                                            (if (and selected 
                                                (not (equal? (node-property-get selected 'node-type) 'action)))
                                              (send selected tab-in-decr)))))))
            
            ;;
            
      )
      ; the graph network editor
      (set! graph-editor-canvas (new graph-editor% 
                                     (parent horizontal-panel-group)
                                     (callback editor-callback)
                                     (font (make-object font% 10 'default 'normal 'normal #f 'default #f))
                                     ))
     
      (send horizontal-panel-group set-percentages '(1/4 3/4))
      (set! graph-editor-list-box widget-choice)
    )
  )
  
) ; end of module


