;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - code-generation.ss                                             #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Lozi Jean-Pierre, 2004 - mailto:jean-pierre@lozi.org             #
;; # Copyright (C) Peter Ivanyi, 2007                                               #
;; # Copyright (C) Laurent Orseau, 2010                                             #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################

(module mreddesigner-code-generation scheme

  (require (lib "class.ss")  
           (lib "mred.ss" "mred")
           (lib "pretty.ss")
           (lib "file.ss")
           (file "mreddesigner-misc.ss")
           (file "mreddesigner-widgets.ss")
           (file "mreddesigner-default-values.ss")
           (file "mreddesigner-callback-edit.ss")
           (file "mreddesigner-callback-code.ss")
  )
  
  (define (prop-to-quote lst)
    (cond
      ((null? lst) '())
      ((list? (cadar lst))
       (cons (list (caar lst) (cons 'quote (cdar lst)))
             (prop-to-quote (cdr lst)))
      )
      (else
       (cons (car lst) (prop-to-quote (cdr lst)))
      )
    )
  )
  
  ; these are the properties that we do not need when PLT-Scheme code is generated
  ; they are either not part of PLT-Scheme or re-generated properly
  (define props-to-remove-write '(hidden collapsed necessary icon pointer 
                                  paint-callback callback type parent children))
  
  (define (make-widget-code id type lst full?)
    (let*
      ((necessary (append '(x y width height) (cadr (assoc 'necessary lst))))
       (props     (if (equal? type 'tab)
                    (assoc-remove lst (cons 'label props-to-remove-write))
                    (assoc-remove lst props-to-remove-write)
                  ))
       (defaults  (widget-default-values type))
      )
      (define (remove-default lst)
        (cond
          ((null? lst) '())
          ; if it is a necessary property
          ((member (caar lst) necessary)
           (cons (car lst) (remove-default (cdr lst)))
          )
          ; if it is a slider there are special considerations for the
          ; stretchable-width and stretchable-height
          ((and (equal? type 'slider)
                (member (caar lst) '(stretchable-width stretchable-height)))
           (cons (car lst) (remove-default (cdr lst)))
          )
          ; if the property is equal to the default then remove it
          ((member (car lst) defaults)
           (remove-default (cdr lst)))
          ; in all other cases preserve this property
          (else
           (cons (car lst) (remove-default (cdr lst)))
          )
        )
      )
      (if full?
        (prop-to-quote props)
        (prop-to-quote (remove-default props))
      )
    )
  )
  
  
  (define (widget-write lst parent full?)
    (cond
      ((null? lst) '())
      ; if it is a container
      ((list? (car lst))
       (let*
         ((full     (send hierarchy-widget get-item (caar lst)))
          (id       (string->symbol (caar lst)))
          (type     (cadr (assoc 'type full)))
          (object   (string->symbol (string-append (symbol->string type) "%")))
          (props    (make-widget-code id type full full?))
          (tab-list '())
          (widget   #f)
          (parent-id (if (equal? type 'tab-panel)
                       (string->symbol (string-append "single-" (caar lst)))
                       id))
         )
         ; tab-panel and tab are special cases
         (cond
           ( (equal? type 'tab-panel)
             (let*
               ; collect the labels of the tabs
               ((choice-labels (map 
                                 (lambda (x) 
                                   (send hierarchy-widget item-property-get (car x) 'label #f)
                                 )
                                 (cdar lst)))
                (choices   (map (lambda (x) (car x)) (cdar lst)))
                (list-id   (string->symbol (string-append (caar lst) "-list")))
                (single-id (string->symbol (string-append "single-" (caar lst))))
                (callback  (list 'lambda '(w e)
                                 (list 'send parent-id 'active-child
                                       (list 'list-ref list-id '(send w get-selection)))))
               )
               (set! widget (list (list id (append '(new tab-panel%)
                                                    (cons (list 'parent parent) props)
                                                    (list (list 'choices (list 'quote choice-labels))
                                                          (list 'callback callback)
                                                    )))
                                  (list single-id (append '(new panel:single%)
                                                           (list (list 'parent id))))
                            ))
               (set! tab-list (list (list list-id (cons 'list 
                                                        (map (lambda (x) (string->symbol x))
                                                             choices)))))
             )
           )
           ( (equal? type 'tab)
             (set! widget (list (list id (append '(new vertical-panel%)
                                                  (cons (list 'parent parent) props)))))
           )
           ; general case
           ( else
             (set! widget (list (list id (append (list 'new object)
                                                 (cons (list 'parent parent) props)))))
           )
         )
         
         (append widget
                 (append (widget-write (cdar lst) parent-id full?)
                         tab-list
                         (widget-write (cdr lst) parent full?))
         )
       )
      )
      ; this is a normal widget
      (else
       (let*
         ((full   (send hierarchy-widget get-item (car lst)))
          (id     (string->symbol (car lst)))
          (type   (cadr (assoc 'type full)))
          (widget (string->symbol (string-append (symbol->string type) "%")))
          (props  (make-widget-code id type full full?))
         )
         (cons (list id (append (list 'new widget) 
                                (cons (list 'parent parent) props)
                                ; there should be a callback
                                (if (member type node-callbacks)
                                  (list (list 'callback (callback-name (car lst))))
                                  '())))
               (widget-write (cdr lst) parent full?)
         )
       )
      )
    )
  )
  
  (define (widget-callback lst)
    (cond
      ((null? lst) '())
      ; if it is a container
      ((list? (car lst))
       (append (widget-callback (cdar lst))
               (widget-callback (cdr lst)))
      )
      ; this is a normal widget
      (else
       (let*
         ((full   (send hierarchy-widget get-item (car lst)))
          (id     (car lst))
          (type   (cadr (assoc 'type full)))
         )
         (if (member type node-callbacks)
           (cons (list (callback-name id) (mreddesigner-callback-generate id))
                 (widget-callback (cdr lst)))
           (widget-callback (cdr lst))
         )
       )
      )
    )
  )
  
  ; *** by Laurent Orseau
    (define (->string x)
      (cond [(string? x) x]
            [(number? x) (number->string x)]
            [(symbol? x) (symbol->string x)]
            [else (format "~a" x)]))
      
    (define (string-append* . l)
      (apply string-append (map ->string l)))

    (define (symbol-append* . l)
      (string->unreadable-symbol (apply string-append* l)))
        
    (define (symbol->keyword sym)
      (string->keyword (symbol->string sym)))
      
    (define (varname->classname vn)
        (symbol-append* vn "-class"))
        
    ;; Replaces the class for the variable with a generic name,
    ;; so that the actual class can be given through an option in the function call.
    (define (variable-reclass v)
      (match v
        [(list vn (list 'new cl args ...))
         (list vn (list* 'new (varname->classname vn) args))]
        [x x]))

    ;; Makes a pair (varname classname) out of a variable,
    ;; only if the variable is an object created with `new'.
    (define (variable->var-class v)
      (match v
        [(list vn (list 'new cl args ...))
         (list (varname->classname vn) cl)]
        [x #f]))

    ;; Given a list of pairs (argument-symbol default-value)
    ;; makes the list of corresponding options with keywords and default value to be added
    ;; to the function head.
    (define (make-options l)
      (apply append 
             (map (lambda (x) (list (symbol->keyword (car x)) x)) 
                  l)))         
  ; ***
    
  (define (show-frames line)
    (cond
      ((null? line) '())
      ((list? (car line))
       (append (show-frames (car line))
               (show-frames (cdr line))))
      (else
       (let*
         ((elem (car line))
          (type (send hierarchy-widget item-property-get elem 'type #f)))
         (if (eq? type 'frame)
           (cons (list 'send (string->symbol elem) 'show #t)
                 (show-frames (cdr line)))
           (show-frames (cdr line))
         )
       ))))
  
  
  ;; This function generates the code from the project-data-global
  ;; and writes it to p-out. The second optional argument is the file
  ;; name.
  (define (generate-code p-out file-name full?)
    (let
      ((name 'test))
      (when file-name
        (let-values 
          (((first second third) (split-path file-name)))
          (let*
            ((file (path->string second))
             (n    (string-length file))
             (ext  (filename-extension file))
             (next (if ext (+ 1 (string-length (bytes->string/utf-8 ext))) 0))
            )
            (set! name (string->symbol (substring file 0 (- n next))))
          )
        )
      )
      ;; First, we write the header, if written to a file.
      (when file-name
        (begin
          (fprintf p-out "~a" 
                   ";;==========================================================================\n")
          (fprintf p-out "~a" 
                   ";;===                Code generated with MrEd Designer 2.2               ===\n")
          (fprintf p-out "~a" 
                   ";;===  http://www.hexahedron.hu/personal/peteri/mreddesigner/index.html  ===\n")
          (fprintf p-out "~a" 
                   ";;==========================================================================\n\n")
          (fprintf p-out ";; File: ~a \n" file-name)
          ))
      (let*
        ; get the hierarchy
        ((hierarchy (send hierarchy-widget get-hierarchy))
         ; assemble the head of the modules
         (module-head (list 'module name 'scheme ;'mzscheme (modif Laurent Orseau)
                            '(require (lib "class.ss") 
                                      (lib "mred.ss" "mred")
                                      ; this is for tab-panel
                                      (lib "framework.ss" "framework"))))
         ; from the widgets create variables
         (variables (widget-write (cdr hierarchy) #f full?))
         (callbacks (widget-callback (cdr hierarchy)))
         ; *** by Laurent Orseau
         (name-init (symbol-append* name "-init"))
         ; Make the (varname classname) pairs and remove non-objects (like tab-list).
         (classes (filter (lambda (x)x) (map variable->var-class variables)))
         ; Redefine variables by changing their class to a classname variable:
         (variables (map variable-reclass variables))
         
         ; writes a module that can be `require'd and manipulated
         ; Default callbacks and classes can be modified by optional arguments to the init function
         (module-code 
           `((provide ,name-init ,@(map car variables))
             ,@(map (lambda (v) `(define ,(car v) #f)) variables)
              (define (,name-init
                       ,@(make-options callbacks)
                       ,@(make-options classes)
                       )
                      ,@(map (lambda (v) (cons 'set! v)) variables)
                      ,@(show-frames hierarchy))
               ;(,name-init)
              ))
         ; ***
        )
        (display (string-append "\n;; Evaluate (" (->string name-init) ") to test the gui.\n\n") p-out)
        (pretty-print (append module-head module-code) p-out)
      )
    )
  )

  
  ;; This function is called when the user chooses to generate the code to a file.
  (provide generate-code-to-file)
  (define (generate-code-to-file  full?)
    (let ((result (put-file "Please choose the output file..." 
                            #f #f #f "" '() 
                            '(("Scheme file (.ss)" "*.ss")
                              ("Scheme file (.scm)" "*.scm")))))
      (when result
        ;; The user did not choose the "Cancel" button.
        (call-with-output-file result 
          (lambda (p-out) (generate-code p-out result  full?)) 
          #:exists 'replace))))
  
  ;; This function is called when the user chooses to generate the code to the console.
  (define (generate-code-to-console full?)
    (generate-code (current-output-port) #f full?))
  
  
  
  ;; This callback is called when the user clicks the "Generate code..." button
  ;; from the toolbox frame.
  (provide generate-code-callback)
  (define (generate-code-callback)
    ;; This callback is called when the user clicks the "OK" button
    (define (ok-button-callback radio-box1 radio-box2 dialog)
      (let*
        ((selection1 (send radio-box1 get-selection))
         (selection2 (send radio-box2 get-selection))
         (full?      (if (= selection2 0) #t #f))
        )
        (send dialog show #f)
        (if (= selection1 0)
          (generate-code-to-file full?)
          (generate-code-to-console full?))))
    
    ;; This callback is called when the user clicks the "Cancel" button
    (define (cancel-button-callback dialog)
      (send dialog show #f))
      ;; We build the whole dialog using a let*
      (let* ((dialog (new dialog% (label "Generate Code...") (parent #f) (x 300) (y 200)))
             ;; The main horizontal pane
             (main-horizontal-pane (new horizontal-pane% 
                                        (parent dialog) 
                                        (border 10)
                                        (alignment '(left top))))
             ;; The app icon
             (message (new message% 
                           (label 'app) 
                           (parent (new vertical-pane% (parent main-horizontal-pane)))))
             ;; The vertical pane
             (vertical-pane (new vertical-pane% 
                                 (parent main-horizontal-pane)
                                 (alignment '(left center))
                                 (min-width 250)))
             ;; The first sub horizontal pane
             (sub-horizontal-pane-1 (new horizontal-pane% 
                                         (parent vertical-pane) 
                                         (border 0) 
                                         (alignment '(left center))))
             ;; The message
             (message (new message% 
                           (label "Where do you want the code to be generated?") 
                           (parent sub-horizontal-pane-1)))
             ;; The radio buttons
             (radio-box1 (new radio-box% 
                              (label #f) 
                              (choices '("To a file" "To the console"))
                              (parent vertical-pane) 
                              (callback void) 
                              (vert-margin 10)))
             
             ;; The second sub horizontal pane
             (sub-horizontal-pane-2 (new horizontal-pane% 
                                         (parent vertical-pane) 
                                         (border 0) 
                                         (alignment '(left center))))
             ;; The message
             (message (new message% 
                           (label "How many properties should be written?") 
                           (parent sub-horizontal-pane-2)))
             ;; The radio buttons
             (radio-box2 (new radio-box% 
                              (label #f) 
                              (choices '("All" "Only required and changed"))
                              (selection 1) ; edit Laurent Orseau
                              (parent vertical-pane) 
                              (callback void) 
                              (vert-margin 10)))
             
             ;; horizontal pane for buttons
             (sub-horizontal-pane-2 (new horizontal-pane% 
                                         (parent vertical-pane) 
                                         (border 0) 
                                         (alignment '(right top))))
             ;; The two buttons
             (okbtn (new button% 
                         (label "OK") 
                         (parent sub-horizontal-pane-2) 
                         (callback (lambda (button control-event) 
                                     (ok-button-callback radio-box1 radio-box2 dialog))) 
                         (min-width 60)))
             (cancelbtn (new button% 
                             (label "Cancel")
                             (parent sub-horizontal-pane-2) 
                             (callback (lambda (button control-event) 
                                         (cancel-button-callback dialog)))
                             (min-width 60))))
        ;; Center the dialog
        (send dialog center)
        ;; Show the modal dialog 
        (send dialog show #t)))
    
) ; end of module

