;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - toolbox-frame.ss                                               #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Lozi Jean-Pierre, 2004 - mailto:jean-pierre@lozi.org             #
;; # Copyright (C) Peter Ivanyi, 2007                                               #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################


(module mreddesigner-toolbox-frame mzscheme
  (require (lib "class.ss")  
           (lib "mred.ss" "mred")
           (file "mreddesigner-widgets.ss")
           (file "mreddesigner-default-values.ss")
           (file "mreddesigner-hierarchy-func.ss")
           (file "mreddesigner-properties-func.ss")
           (file "mreddesigner-preview.ss")
           (file "mreddesigner-file.ss")
           (file "mreddesigner-code-generation.ss")
           (file "mreddesigner-help.ss")
           (file "mreddesigner-callback.ss")
           (file "mreddesigner-tooltip.ss")
  )
  
  (provide project-init)
  (define (project-init)
    (send hierarchy-widget clear)
    (send hierarchy-widget 
          item-append #f "project" 
          (widget-default-values 'project)
          (icon-get 'project)
          #t #f)
    ; update the buttons for the hierarchy
    (update-hierarchy-buttons)
    (changed! #t)
  )
  
  ;;; ----------------------------------------------------------------
  ;;; callback functions
  ;;; ----------------------------------------------------------------
    
  ;; Function called when the user chooses "File->New"
  (define (files-new)
    ; check whether to save a modified project
    (if (not (changed?))
      (let
        ((save? (message-box "Warning" 
                             (string-append "Do you want to save the current project\n"
                                            "before creating a new one?")
                             #f '(yes-no caution))))
        (if (eq? 'yes save?)
          (files-save))))
    ; hide all windows
    (preview-show-frames (send hierarchy-widget get-hierarchy) #f)
    ; create a new project and update the hierachy widget
    (project-init)
    ; update the property windows
    (send properties-panel empty)
    ; no need to update the graph editor
    ; when the hierarchy widget is cleared the graph widget will be also cleared
  )

  ;; Function called when the user chooses "File->Open"
  (define (files-open)
    (let 
      ((result (if (not (changed?))
                 (message-box/custom "Warning" 
                                     (string-append 
                                         "Do you want to save the current project\n"
                                         "before opening a new one?")
                                     "Yes" "No" "Cancel" #f 
                                     '(default=1 number-order caution) 
                                     3)
                 2)))
      (if (= 1 result) (files-save))
      (if (not (= 3 result))
        (let ((result-file (get-file "Open..." #f #f #f "*.med" '() 
                                     '(("MrEd Designer Project (.med)" "*.med"))) ))
          (if result-file
            (let ((result (path->string result-file)))
              (if (file-exists? result)
                (if (mreddesigner-read result)
                  (callback-graph-read result)
                )
                (message-box "Error" "The file does not exist." #f '(ok stop)))))))))

  
  ;; Function called when the user chooses "File->Save As..."
  (define (files-save-as)
    (let ((result-file (put-file "Save as..." #f #f #f "*.med" '() 
                                 '(("MrEd Designer Project (.med)" "*.med"))) ))
      (if result-file
        (let ((result (path->string result-file)))
          ;; We update the project label and path
          (let-values 
            (((base name must-be-dir?)(split-path result)))
            (let
              ((label (car (regexp-match "[^.]+" (path->string name)))))
              (send hierarchy-widget item-property-set! "project" 'label label #f)
              (send hierarchy-widget item-property-set! "project" 'path result #f)
            )
          )
          ; then we write out the actual file
          (mreddesigner-write result)
          ; write out the graph network
          (callback-graph-write result)
          ; update the hierarchy buttons
          (update-hierarchy-buttons)
          ; notify properties window
          (send properties-panel empty)
          
          (changed! #t)
        )
      )
      result-file))

  ;; Function called when the user chooses "File->Save"
  (define (files-save)
    (let ((path (send hierarchy-widget item-property-get "project" 'path #f)))
      (if (not (string=? path ""))
        (if (file-exists? path)
          (begin
            (mreddesigner-write path)
            (callback-graph-write path)
            ;; Then we update the no-change-since-last-save variable...
            (changed! #t)
            path
          )
          (begin
            (message-box "Error" 
                         (string-append
                           "Unable to save the project because the file does not exist anymore.\n"
                           "Please check it has not been either deleted, moved or renamed.")
                         #f '(ok stop))
            (files-save-as)))
      (files-save-as))))


  (define (files-quit-final)
    ;; we hide all previews
    (preview-show-frames (send hierarchy-widget get-hierarchy) #f)
    ;; hide the hierarchy window and properties window
    (if (send hierarchy-frame is-shown?)
      (send hierarchy-frame show #f)
    )
    (if (send properties-frame is-shown?)
      (send properties-frame show #f)
    )
    (if (send callback-graph-frame is-shown?)
      (send callback-graph-frame show #f)
    )
    (if (send toolbox-frame is-shown?)
      (send toolbox-frame show #f)
    )
  )
  
  ;; Function called when the user chooses "File->Quit"
  (define (files-quit cancel)
    (let 
      ((result (if (not (changed?))
                 (if cancel
                   (message-box/custom "Warning" 
                                       "Would you like to save changes before quitting?"
                                       "Yes" "No" "Cancel" #f '(default=1 number-order caution) 3)
                   (message-box/custom "Warning" 
                                       "Would you like to save changes before quitting?"
                                       "Yes" "No" #f #f '(default=1 number-order caution) 2))
                 2)))
      (cond
        ((= 1 result) ; Yes
         (if cancel
           (if (files-save)
             (files-quit-final)
             (message-box "Warning" 
                          (string-append
                               "Unable to save the project.\n"
                               "Program is not quitting.")
                             #f '(ok))
           )
           (begin
            (files-save)
             (files-quit-final)
           )
          )
         )
        ((= 2 result)
         (files-quit-final)))))

  ;;; ----------------------------------------------------------------
  ;;; widgets
  ;;; ----------------------------------------------------------------
  
(toolbox-frame-init (lambda () (files-quit #f)))
  
  
;; Main menu bar definition...
(define toolbox-frame-menu (new menu-bar%
                                (parent toolbox-frame)))

;; File menu definition...
(define toolbox-frame-file-menu (new menu%
                                    (label "File") 
                                    (parent toolbox-frame-menu) 
                                    (help-string "File menu")))

;; "New Project..." item definition...
(new menu-item%
     (label "&New Project...")
     (shortcut #\N)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (files-new))))

;; "New Project..." item definition...
(new menu-item%
     (label "&Open Project...")
     (shortcut #\O)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (files-open))))

;; "New Project..." item definition...
(new separator-menu-item% (parent toolbox-frame-file-menu))

;; "Save" item definition...
(new menu-item%
     (label "&Save")
     (shortcut #\S)
     (parent toolbox-frame-file-menu)
     (callback (lambda (menu-item control-event) (files-save))))

;; "Save as..." item definition...
(new menu-item%
     (label "&Save as...")
     (parent toolbox-frame-file-menu)
     (callback (lambda (menu-item control-event) (files-save-as))))

;; "Export..." item definition...
(new menu-item%
     (label "&Export as a Scheme File...")
     (shortcut #\E)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) 
                 (generate-code-to-file #t)
               )))

(new separator-menu-item% (parent toolbox-frame-file-menu))

;; "Quit" item definition...
(new menu-item%
     (label "E&xit")
     (shortcut #\Q)
     (parent toolbox-frame-file-menu)
     (callback (lambda (item event) (toolbox-frame-file-menu-on-quit))))

;; Edit menu definition...
(define toolbox-frame-edit-menu (new menu%
                                    (label "Edit") 
                                    (parent toolbox-frame-menu) 
                                    (help-string "Edit menu")))

;; "Copy" item definition...
(define toolbox-frame-edit-copy (new menu-item%
                                     (label "&Copy")
                                     (shortcut #\C)
                                     (parent toolbox-frame-edit-menu)
                                     (callback (lambda (item event)
                                                 (let
                                                   ((selected (send hierarchy-widget get-selected)))
                                                   (if selected
                                                     (clipboard-copy selected #f)
                                                   )
                                                 )
                                               ))))

;; "Copy with children" item definition...
(define toolbox-frame-edit-copy-c (new menu-item%
                                       (label "Copy &with children")
                                       (shortcut #\w)
                                       (parent toolbox-frame-edit-menu)
                                       (callback (lambda (item event) 
                                                   (let
                                                     ((selected (send hierarchy-widget get-selected)))
                                                     (if selected
                                                       (clipboard-copy selected #t)
                                                     )
                                                   )
                                                 ))))


;; "Cut" item definition...
(define toolbox-frame-edit-cut (new menu-item%
                                    (label "C&ut")
                                    (shortcut #\X)
                                    (parent toolbox-frame-edit-menu)
                                    (callback (lambda (item event)
                                                (let
                                                  ((selected (send hierarchy-widget get-selected)))
                                                  (if selected
                                                    (clipboard-cut selected)
                                                  )
                                                )
                                              ))))

;; "Paste" item definition...
(define toolbox-frame-edit-paste (new menu-item%     
                                      (label "&Paste")
                                      (shortcut #\V)
                                      (parent toolbox-frame-edit-menu)
                                      (callback (lambda (item event)
                                                  (let
                                                    ((selected (send hierarchy-widget get-selected)))
                                                    (if selected
                                                      (clipboard-paste selected)
                                                    )
                                                  )
                                                ))))

;(send toolbox-frame-edit-paste enable #f)
(define toolbox-frame-edit-delete (new menu-item%     
                                       (label "&Delete")
                                       (shortcut #\D)
                                       (parent toolbox-frame-edit-menu)
                                       (callback (lambda (item event)
                                                   (let
                                                     ((selected (send hierarchy-widget get-selected)))
                                                     (if (and selected (not (equal? selected "project")))
                                                       (send hierarchy-widget item-delete selected)
                                                     )
                                                   )
                                                 ))))

;; Windows menu definition...
(define toolbox-frame-windows-menu (new menu%
                                        (label "Windows") 
                                        (parent toolbox-frame-menu) 
                                        (help-string "Windows menu")))

;; "Hierarchy frame" item definition...
(define properties-frame-windows-hierarchy-frame-menu-item 
  (new checkable-menu-item%
       (label "Hierarchy frame")
       (parent toolbox-frame-windows-menu)
       (callback (lambda (item event) 
                   (if (send item is-checked?)
                     (begin
                       (send hierarchy-frame show #t)
                       (send item check #t))
                     (begin
                       (send hierarchy-frame show #f)
                       (send item check #f)))))))

;; Perform necessary updates...
(send properties-frame-windows-hierarchy-frame-menu-item enable #t)
(send properties-frame-windows-hierarchy-frame-menu-item check #t)

;; "Properties frame" item definition...
(define properties-frame-windows-properties-frame-menu-item 
  (new checkable-menu-item%
       (label "Properties frame")
       (parent toolbox-frame-windows-menu)
       (callback (lambda (item event) 
                   (if (send item is-checked?)
                     (begin
                       (send properties-frame show #t)
                       (send item check #t))
                     (begin
                       (send properties-frame show #f)
                       (send item check #f)))))))

;; Perform necessary updates...
(send properties-frame-windows-properties-frame-menu-item enable #t)
(send properties-frame-windows-properties-frame-menu-item check #t)


;; "Graph Editor frame" item definition...
(define properties-frame-windows-graph-frame-menu-item 
  (new checkable-menu-item%
       (label "Callback Graph frame")
       (parent toolbox-frame-windows-menu)
       (callback (lambda (item event) 
                   (if (send item is-checked?)
                     (begin
                       (send callback-graph-frame show #t)
                       (send item check #t))
                     (begin
                       (send callback-graph-frame show #f)
                       (send item check #f)))))))

;; Perform necessary updates...
(send properties-frame-windows-graph-frame-menu-item enable #t)
(send properties-frame-windows-graph-frame-menu-item check #f)


;; Help menu definition...
(define toolbox-frame-help-menu (new menu%
                                     (label "Help") 
                                     (parent toolbox-frame-menu) 
                                     (help-string "Help menu")))

;;; "Online Help..." item definition...
;(new menu-item%
;     (label "Online &Help...")
;     (shortcut #\H)
;     (parent toolbox-frame-help-menu)
;     (callback (lambda (item event) 
;                 (void)
;                 ;(help-online-help)
;               )))
;
;;; "MrEd Help (from PLT)..." item definition...
;(new menu-item%
;     (label "&MrEd Help...")
;     (parent toolbox-frame-help-menu)
;     (callback (lambda (item event) 
;                 ;(help-mred-help)
;                 (void)
;               )))
;
;(new separator-menu-item% (parent toolbox-frame-help-menu))

;; "About MrEd Designer..." item definition...
(new menu-item%
     (label "&About MrEd Designer...")
     (shortcut #\?)
     (parent toolbox-frame-help-menu)
     (callback (lambda (item event) 
                 (help-about-dialog)
               )))
    
;; The vertical pane
(define toolbox-frame-vertical-pane 
  (new vertical-pane% 
       (parent toolbox-frame) 
       (alignment '(right top)) 
       (border 3)))

;; The first group box panel
(define toolbox-frame-first-gbp 
  (new group-box-panel% 
       (label "Containers") 
       (parent toolbox-frame-vertical-pane)))

;; The first horizontal pane
(define toolbox-frame-first-horizontal-pane 
  (new horizontal-pane% 
       (parent toolbox-frame-first-gbp)))

;; We fill it
(define (toolbox-frame-make-first-buttons list)
  (if (not (null? list))
    (begin
      (new mred-button-tooltip%
           (label (make-object bitmap% 
                    (build-path "images" "icons" "24x24" (list-ref (car list) 1))
                    'png (make-object color% 255 255 255)))
           (tooltip-text (list-ref (car list) 2))
           (parent toolbox-frame-first-horizontal-pane)
           (style '(border))
           (callback (lambda (button event) ((list-ref (car list) 0) button event))))
      (toolbox-frame-make-first-buttons (cdr list)))))

;; The second group box panel
(define toolbox-frame-second-gbp 
  (new group-box-panel% 
       (label "Controls") 
       (parent toolbox-frame-vertical-pane)))

;; The second horizontal pane
(define toolbox-frame-second-horizontal-pane 
  (new horizontal-pane% 
       (parent toolbox-frame-second-gbp)))

;; We fill it
(define (toolbox-frame-make-second-buttons list)
  (if (not (null? list))
    (begin
      (new mred-button-tooltip%
           (label (make-object bitmap% 
                    (build-path "images" "icons" "24x24" (list-ref (car list) 1))
                    'png (make-object color% 255 255 255))) 
           (tooltip-text (list-ref (car list) 2))
           (parent toolbox-frame-second-horizontal-pane)
           (style '(border))
           (callback (lambda (button event) ((list-ref (car list) 0) button event))))
      (toolbox-frame-make-second-buttons (cdr list)))))

;; Callback called when the user quits the application :
(define (toolbox-frame-file-menu-on-quit)
  (files-quit #t))
 
;; Callback called when user clicks the "frame" button.
;; It will also call the default callback.
(define (toolbox-frame-button-frame-callback button event)  
  (let ((selected-element (send hierarchy-widget get-selected)))
    ;; If the parent is another frame...
    (cond ((and selected-element 
                (eq? (send hierarchy-widget item-property-get selected-element 'type #f) 'frame))
           ;; Then call the default callback...
           (default-callback 'frame frame-default-values button))
          ;; If the project root is selected in the hierarchy widget...
          ((or (not selected-element)
               (eq? (send hierarchy-widget item-property-get selected-element 'type #f) 'project))
            ;; ...create the frame...
            (let*
             ((item (make-widget-preview #f frame-default-values))
              (item-frame (append item (list '(hidden #f))))
             )
             ;; ...add the frame to the project data...
             (send hierarchy-widget 
                   item-append 
                   "project"
                   (car item-frame)
                   (cdr item-frame)
                   (icon-get 'frame)
                   #t #f)
             
             ;; ...update the no-change-since-last-save variable...
             (changed! #f)
             
             ;; ...show the frame...
             (send (cadr (assoc 'pointer (cdr item))) show #t)
             
             ;; ...and refresh the button.
             (send button refresh)))
          
          ;; Otherwise...
          (else
           ;; ...display an error.
           (message-box "Error" 
                   (string-append "You cannot create a frame because the\n"
                                  "currently selected element\n"
                                  "is neither a frame nor the project root.") 
                   toolbox-frame '(ok stop))))))

;; Callback called when user clicks the "pane" button :
(define (toolbox-frame-button-pane-callback button event)
  (default-callback 'pane pane-default-values button))

;; Callback called when user clicks the "horizontal pane" button :
(define (toolbox-frame-button-horizontal-pane-callback button event)
  (default-callback 'horizontal-pane horizontal-pane-default-values button))

;; Callback called when user clicks the "vertical pane" button :
(define (toolbox-frame-button-vertical-pane-callback button event)
  (default-callback 'vertical-pane vertical-pane-default-values button))

;; Callback called when user clicks the "panel" button :
(define (toolbox-frame-button-panel-callback button event)
  (default-callback 'panel panel-default-values button))

;; Callback called when user clicks the "horizontal panel" button :
(define (toolbox-frame-button-horizontal-panel-callback button event)
  (default-callback 'horizontal-panel horizontal-panel-default-values button))

;; Callback called when user clicks the "vertical panel" button :
(define (toolbox-frame-button-vertical-panel-callback button event)
  (default-callback 'vertical-panel vertical-panel-default-values button))

;; Callback called when user clicks the "group box panel" button :
(define (toolbox-frame-button-group-box-panel-callback button event)
  (default-callback 'group-box-panel group-box-panel-default-values button))

;; Callback called when user clicks the "tab panel" button :
(define (toolbox-frame-button-tab-panel-callback button event)
  (default-callback 'tab-panel tab-panel-default-values button))

;; Callback called when user clicks the "tab" button :
(define (toolbox-frame-button-tab-callback button event)
  (default-callback 'tab tab-default-values button))

;; Callback called when user clicks the "grow box spacer pane" button :
(define (toolbox-frame-button-grow-box-spacer-pane-callback button event)
  (default-callback 'grow-box-spacer-pane grow-box-spacer-pane-default-values button))

;; Callback called when user clicks the "message" button :
(define (toolbox-frame-button-message-callback button event)
  (default-callback 'message message-default-values button))

;; Callback called when user clicks the "button" button :
(define (toolbox-frame-button-button-callback button event)
  (default-callback 'button button-default-values button))

;; Callback called when user clicks the "radio-box" button :
(define (toolbox-frame-button-radio-box-callback button event)
  (default-callback 'radio-box radio-box-default-values button))

;; Callback called when user clicks the "check-box" button :
(define (toolbox-frame-button-check-box-callback button event)
  (default-callback 'check-box check-box-default-values button))

;; Callback called when user clicks the "slider" button :
(define (toolbox-frame-button-slider-callback button event)
  (default-callback 'slider slider-default-values button))

;; Callback called when user clicks the "gauge" button :
(define (toolbox-frame-button-gauge-callback button event)
  (default-callback 'gauge gauge-default-values button))

;; Callback called when user clicks the "list-box" button :
(define (toolbox-frame-button-list-box-callback button event)
  (default-callback 'list-box list-box-default-values button))

;; Callback called when user clicks the "choice" button :
(define (toolbox-frame-button-choice-callback button event)
  (default-callback 'choice choice-default-values button))

;; Callback called when user clicks the "text-field" button :
(define (toolbox-frame-button-text-field-callback button event)
  (default-callback 'text-field text-field-default-values button))

;; Callback called when user clicks the "canvas" button :
(define (toolbox-frame-button-canvas-callback button event)
  (default-callback 'canvas canvas-default-values button))
 
;; This is a default callback for each button
(define (default-callback type default-values btn)
  (let ((selected-id (send hierarchy-widget get-selected)))
    (if selected-id
      (let
        ((selected-type (send hierarchy-widget item-property-get selected-id 'type #f)))
        (cond
;          ( (and (equal? type 'tab-panel) (equal? selected-type 'tab))
;            (message-box "Error" 
;                         (string-append "The tab-panel widget cannot be created\n"
;                                        "because it cannot be added to a tab.") 
;                         toolbox-frame '(ok stop))
;          )
          ( (and (equal? type 'tab) (not (equal? selected-type 'tab-panel)))
            (message-box "Error" 
                         (string-append "The tab widget cannot be created\n"
                                        "because the currently selected element\n"
                                        "is not a tab-panel.") 
                         toolbox-frame '(ok stop))
          )
          ( (and (not (equal? type 'tab)) (equal? selected-type 'tab-panel))
            (message-box "Error" 
                         (string-append "This widget cannot be created\n"
                                        "because it cannot be added to a tab-panel.")
                         toolbox-frame '(ok stop))
          )
          ; ----------------------------------------------------------
          ; add a tab to the tab-panel
          ; ----------------------------------------------------------
          ( (and (equal? type 'tab) (equal? selected-type 'tab-panel))
            ;; Create the tab
            (let*
              ; get the parent object, the tab-panel
              ( (item (make-widget-preview selected-id default-values)) )
              ;; add the tab to the hierarchy widget
              (send hierarchy-widget 
                    item-append
                    selected-id
                    (car item)
                    (cdr item)
                    (icon-get type)
                    (container? type)
                    #f)

              ;; At least one change have been made
              (changed! #f)
              
              ;;; FIXME: this is another hick-up in MrEd
              ; when a new tab is added to a tab-panel the border style of the
              ; tab-panel always defaults to have a border, therefore
              ; I update the full window, so the program will consider the actual
              ; setting for the border style
              (preview-show-frames (send hierarchy-widget get-hierarchy) #f)
              (preview-update-all)
              (preview-show-frames (send hierarchy-widget get-hierarchy) #t)

              ;; Refresh the button because of a bug on MacOS X
              (send btn refresh)
            )
          )
          ; ----------------------------------------------------------
          ; general adding with some exception
          ; ----------------------------------------------------------
          ( (and (not (equal? type 'tab))
                 (not (equal? selected-type 'tab-panel))
                 (container? selected-type))
            ;; Create the element
            (let*
              ((item       (make-widget-preview selected-id default-values))
               (item-frame (if (equal? type 'frame) (append item (list '(hidden #f))) item))
              )
              ;; add the item to the hierarchy widget
              (send hierarchy-widget 
                    item-append
                    selected-id
                    (car item-frame)
                    (cdr item-frame)
                    (icon-get type)
                    (container? type)
                    #f)
            
              ;; If the element is a frame...
              (if (equal? type 'frame)
                (begin
                  ;; ...show it!
                  (send (cadr (assoc 'pointer (cdr item))) show #t)
                )
              )

              ;; At least one change have been made
              (changed! #f)
              
              ;; Refresh the button because of a bug on MacOS X
              (send btn refresh)
            )
          )
          ; ----------------------------------------------------------
          ; ----------------------------------------------------------
          (else
            (message-box "Error" 
                         (string-append "You cannot instantiate this widget\n"
                                        "because the currently selected element\n"
                                        "is not a container.") 
                         toolbox-frame '(ok stop)))
        )
      )
      (message-box "Error" "No element is selected" toolbox-frame '(ok stop))
    )
  )
)

;; Lists containing callbacks and corresponding pictures
;; for the toolbox frame buttons...
(define toolbox-frame-buttons-first-list
  (list (list toolbox-frame-button-frame-callback  "frame.png" "frame")
        (list toolbox-frame-button-pane-callback  "pane.png" "pane")
        (list toolbox-frame-button-horizontal-pane-callback  "horizontal-pane.png" "horizontal-pane")
        (list toolbox-frame-button-vertical-pane-callback  "vertical-pane.png" "vertical-pane")
        (list toolbox-frame-button-panel-callback  "panel.png" "panel")
        (list toolbox-frame-button-horizontal-panel-callback  "horizontal-panel.png" "horizontal-panel")
        (list toolbox-frame-button-vertical-panel-callback  "vertical-panel.png" "vertical-panel")
        (list toolbox-frame-button-grow-box-spacer-pane-callback  "grow-box-spacer-pane.png" "box-pane")
        (list toolbox-frame-button-group-box-panel-callback  "group-box-panel.png" "group-box-panel")
        (list toolbox-frame-button-tab-panel-callback  "tab-panel.png" "tab-panel")
        (list toolbox-frame-button-tab-callback "tab.png" "tab")
        ))

(define toolbox-frame-buttons-second-list
  (list (list toolbox-frame-button-message-callback  "message.png" "message")
        (list toolbox-frame-button-radio-box-callback  "radio-box.png" "radio-box")
        (list toolbox-frame-button-check-box-callback  "check-box.png" "check-box")
        (list toolbox-frame-button-button-callback  "button.png" "button")
        (list toolbox-frame-button-canvas-callback  "canvas.png" "canvas")
        (list toolbox-frame-button-choice-callback  "choice.png" "choice")
        (list toolbox-frame-button-gauge-callback  "gauge.png" "gauge")
        (list toolbox-frame-button-list-box-callback  "list-box.png" "list-box")
        (list toolbox-frame-button-slider-callback  "slider.png" "slider")
        (list toolbox-frame-button-text-field-callback  "text-field.png" "text-field")))

;; We add the "Generate Code..." button...
(new button%
     (label "Generate Code...") 
     (parent toolbox-frame-vertical-pane)
     (min-width 150)
     (callback (lambda (button event)
                 (generate-code-callback)
               ))
)

; generate the two lines of buttons
(toolbox-frame-make-first-buttons toolbox-frame-buttons-first-list)
(toolbox-frame-make-second-buttons toolbox-frame-buttons-second-list)

; create the hierarchy frame
(hierarchy-frame-init (lambda () 
                        (send properties-frame-windows-hierarchy-frame-menu-item check #f)
                      )
                      hierarchy-up-func
                      hierarchy-up-one-func
                      hierarchy-down-one-func
                      hierarchy-down-func
                      hierarchy-delete-func
                      hierarchy-hide-func
                      hierarchy-show-func
                      hierarchy-widget-func
                      hierarchy-widget-key-func
)

; create the property frame
(properties-frame-init (lambda ()
                         (send properties-frame-windows-properties-frame-menu-item check #f)
                       )
                       properties-func
)

; graph editor
(callback-graph-frame-init (lambda ()
                             (send properties-frame-windows-graph-frame-menu-item check #f)
                           )
                           callback-graph-frame-create
)

  
) ; end of module

